﻿using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using System;
using System.Reflection;
using System.Threading.Tasks;
using HealthMonitor.Core.Operator;
using HealthMonitor.Util.Common.Operator;
using HealthMonitor.Util.Entities.Base;
using HealthMonitor.Util.Entities.Interfaces;

namespace HealthMonitor.Core.Pipeline.Aop.Query
{
	public class TryGetRefManagerValue:AopValueBase
	{
		private readonly static MethodInfo M_TaskFromResult = typeof(Task).GetMethod(nameof(Task.FromResult))!;

		public override async Task Invoke(CacheInterceptorContext context)
		{
			var entityType = context.EntityType;
			var returnType = context.ReturnType;
			var isAsync = context.IsAsyncMethod;
			var isEnumerable = context.IsReturnEnumerable;
			var param = context.AopContext.Parameters[0];

			if (entityType == null || returnType == null
				|| isEnumerable //从待插入队列中读取实体，不支持读取多个实体（列表）
				|| param is not string)
			{
				await InvokeNextAsync(context);
				return;
			}

			//计算key值
			var entity = Activator.CreateInstance(entityType);
			var method = entityType.GetMethod(nameof(IEntity.GetUniqueKey));
			var key = method?.Invoke(entity, new[] { param }) + "";

			if (string.IsNullOrEmpty(key))
			{
				await InvokeNextAsync(context);
				return;
			}

			OperateCacheItem cacheItem = null!;
			bool isDeleted = false;
			/*
			 * 注意这里是有并发的可能性（负责增删改的后台线程会执行2个步骤：a、从待插入列表中获取并删除实体数据，b、获取到的实体数据入库，当查询发生在a与b之间就会出现该并发情况）
			 * 因为该并发情况发生几率很低，而且更重要的一点是查询的性能要求比较高尽量不能加互斥锁，所以该并发允许存在，不作修改
			 */
			IOperatorManager operatorManager = null!;
			//if (entityType.IsSubclassOf(typeof(GpsCardEntityBase))) operatorManager = context.AopContext.ServiceProvider.GetService<IGpsCardOperatorManager>();
			//else if (entityType.IsSubclassOf(typeof(TelpoCommonEntityBase))) operatorManager = context.AopContext.ServiceProvider.GetService<ITelpoCommonOperatorManager>();

			if (operatorManager != null)
			{
				cacheItem = operatorManager.GetDelayOperator(DelayOperateTypeEnum.Insert, key)!;
				if (cacheItem == null) cacheItem = operatorManager.GetDelayOperator(DelayOperateTypeEnum.Update, key)!;
				if (cacheItem == null && operatorManager.GetDelayOperator(DelayOperateTypeEnum.Delete, key) != null) isDeleted = true;
				//从pending状态的数据中检索
				if (cacheItem == null && operatorManager.PendingInsertOperators != null) operatorManager.PendingInsertOperators.TryGetValue(key, out cacheItem!);
				if (cacheItem == null && operatorManager.PendingUpdateOperators != null) operatorManager.PendingUpdateOperators.TryGetValue(key, out cacheItem!);

				if (cacheItem != null)
				{
					context.Logger.LogInformation($"击中操作缓存{key}");

					entity = JsonConvert.DeserializeObject(cacheItem.JsonData!, entityType) as IEntity;  //强制转换as将导致返回null或者IEntity实例，然后赋值给object类型
					if (entity != null)
					{
						if (isAsync) //&& methodReturnType == typeof(Task<>).MakeGenericType(returnType))
						{
							//反射获取Task<>类型的返回值，相当于Task.FromResult(value);
							context.AopContext.ReturnValue = M_TaskFromResult.MakeGenericMethod(returnType).Invoke(null, new[] { entity });
							return;
						}

						context.AopContext.ReturnValue = entity;
						return;
					}
				}
			}

			if (isDeleted)
			{
				if (isAsync)
				{
					//反射获取Task<>类型的返回值，相当于Task.FromResult(value);
					context.AopContext.ReturnValue = M_TaskFromResult.MakeGenericMethod(returnType).Invoke(null, new object[] { null! });
					return;
				}

				context.AopContext.ReturnValue = null;
				return;
			}

			await InvokeNextAsync(context);
		}
	}
}
